% INFERMFA infers responsibilities and posterior means of components of MFA.
%
% [RR, ZZ] =  infermfa(XX,LX,PX,MX,Pi);
%

function [RR, ZZ] =  infermfa(XX,LX,PX,MX,Pi);

[DD NN] = size(XX);
KK      = size(MX,2);
FF      = size(LX,2); % size(Lh) = [D F N]

Phi   = diag(1./PX);

RR = zeros(KK,NN);
ZZ = zeros(FF,NN,KK);

oneNN = ones(1,NN);
oneKK = ones(1,KK);

for kk=1:KK,
  lh = LX(:,:,kk);
  mu = MX(:,kk);
  xx = XX-mu(:,oneNN);
  cm = Phi - Phi*lh/(eye(FF)+lh'*Phi*lh)*lh'*Phi;

  %ZZ(:,:,kk) = (lh'*lh)\lh'*xx; % ok
  ZZ(:,:,kk) = (lh'*cm)*xx;        % correct

  RR(kk,:) = log(Pi(kk)) + .5*log(det(cm)) - .5*sum(xx.*(cm*xx),1); % correct
  %RR(kk,:) = log(Pi(kk)) - .5*sum(xx.*(cm*xx),1);                   % ok
  %RR(kk,:) = -.5*sum(ZZ(:,:,kk).^2,1) + log(Pi(kk));                % bad
end;
mm = max(RR,[],1);
RR = exp(RR - mm(oneKK,:));
mm = sum(RR,1);
RR = RR./mm(oneKK,:);
ZZ = permute(ZZ,[1 3 2]);

