
function [LX,MX,PX,Pi] = mfa(XX,FF,KK,tol,maxiter,minstd,LX,MX,PX,Pi);

[DD NN] = size(XX);
oneNN = ones(1,NN);
oneDD = ones(1,DD);
oneKK = ones(1,KK);
oneFF = ones(1,FF);
eyeFF = eye(FF);

% Estimate minimum variance allowed.
%mm = zeros(DD,1);
%for ii=1:DD,
%  mm(ii) = min(diff(sort(XX(ii,:))));
%end;
%minstd = min(mm)/2;
minvar = minstd^2;

% Initialize
if nargin==6,
  mm = mean(XX,2);
  ss = cov(XX');
  cc = chol(ss);
  MX = cc' * randn(DD,KK) + mm(:,oneKK);
  LX = minstd * randn(DD,FF,KK);
  PX = 2*mean(diag(cc))*oneDD';
  Pi = oneKK'/KK;
  clear mm ss cc
end;

XX2 = XX.^2;
const = -DD/2*log(2*pi);
lik = -inf;
rr  = zeros(KK,NN);
czz = zeros(FF,FF,KK);
zz  = zeros(FF,NN,KK);

tic
for ii=1:maxiter,

  %plot(XX(1,:),XX(2,:),'.');
  %plotmfa2(LX,PX,MX);
  %drawnow;
  %pause;

  % E step
  pi = 1./PX;
  for kk=1:KK,
    l        = LX(:,:,kk);
    ltpi     = (pi(:,oneFF).*l)';
    ltpil    = ltpi*l;
    iltpil   = eyeFF + ltpil;
    cc       = chol(iltpil);
    cci      = inv(cc);
    covz     = cci*cci';
    delta    = XX-MX(:,kk*oneNN);
    meanz    = ((eyeFF-ltpil*covz)*ltpi)*delta;

    czz(:,:,kk) = covz;
    zz(:,:,kk)  = meanz;
    rr(kk,:)    = -.5*(pi'*(delta.*delta) - sum(meanz.*(iltpil*meanz),1)) - ...
                  sum(log(diag(cc)));
  end;

  rr = rr + const + .5*sum(log(pi));
  mm = max(rr,[],1);
  rr = exp(rr-mm(oneKK,:));
  ss = sum(rr,1);
  rr = rr./ss(oneKK,:);

  oldlik = lik;
  lik = sum(mm+log(ss));
  fprintf(1,'iter%3d lik%f     \n',ii,lik);
  if abs(oldlik-lik)<tol, break; end;

  % M step
  Pi = mean(rr,2);
  PX = 0;
  for kk=1:KK,
    r    = rr(kk,:);
    z    = zz(:,:,kk);
    rz   = z.*r(oneFF,:);
    sr   = sum(r);
    srz  = sum(rz,2);
    srxz = XX*rz';
    srx  = XX*r';
    m1   = [srxz srx];
    m2   = [sr*czz(:,:,kk) + z*rz' srz; srz' sr];
    m1   = m1/m2;
    LX(:,:,kk) = m1(:,1:FF);
    MX(:,kk) = m1(:,FF+1);
    PX = PX + XX2*r' - sum(LX(:,:,kk).*srxz,2) - MX(:,kk).*srx;
  end;
  PX = max(minvar,PX/NN);

end;
time=toc;
fprintf(1,'   [time %f (%f/iter)\n',time,time/ii);
