function llelineviewer(y1,y2,P);

% LLELINEVIEWER -- Find closest interpolants between two clicked points 
%   linearly by LLE.  Do this by finding point closest to segment joining
%   the two end points.
%
% P should be a structure with:
%
%   hv          handle to image object where images are shown.
%   hf          handle to line object to trace interpolants on finder.
%
%   snap        1 if points snap to closest data points. 0 otherwise.
%   dims        [d1 d2] corresponding to dimension of x,y axes in finder.
%   idims       dimensions along which to do interpolation.
%
%   imsize      size of images (prod(imsize)=DD).
%   numim       number of interpolants (including end points).
%   numrep      nuber of repeats for animation.
%   montage     1 if interpolants shown as horizontal montage. 0 for animation.
%   crange      Range of pixel intensities for images.
%
%   XX,YY       Data and embeddings produced by LLE.

dims  = P.dims;
idims = P.idims;
if P.snap,
  ii  = xknn(P.YY(dims,:),[y1 y2],1);
  y1  = P.YY(dims,ii(1));
  y2  = P.YY(dims,ii(2));
  iy1 = P.YY(idims,ii(1));
  iy2 = P.YY(idims,ii(2));
else
  idims = dims;
  iy1   = y1;
  iy2   = y2;
end;

[d2 proj] = projline(iy1,iy2,P.YY(idims,:));
ii = find(proj>=0 & proj<=1);
numim = min(length(ii),P.numim);

[d2 jj] = sort(d2(ii));
ii = ii(jj(1:numim));
[proj jj] = sort(proj(ii));
ii = ii(jj);
YY = P.YY(dims,ii);
XX = P.XX(:,ii);

if P.montage,
  rehinton(P.hv,XX,[P.imsize 1 P.numim],P.crange);
else
  for ii=1:P.numrep,
    for jj=[1:P.numim P.numim:-1:1],
      rehinton(P.hv,XX(:,jj),P.imsize,P.crange);
      drawnow;
    end;
  end;
end;

set(P.hf,'xdata',YY(1,:),'ydata',YY(2,:));
